﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Threading;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;

namespace ConverterSlow
{
    public class VM : ViewModelBase
    {
        #region Constructor

        /// <summary>
        /// Constructor
        /// </summary>
        public VM()
        {
            Setup();
            LoadedCommandExecuted();
        }

        #endregion

        #region Properties


        private ObservableCollection<ValicMricMap> _allItems;
        /// <summary>
        /// Collection of VALIC MRIC Map items
        /// </summary>
        public ObservableCollection<ValicMricMap> AllItems
        {
            get { return _allItems; }
            set
            {
                if (_allItems != value)
                {
                    _allItems = value;
                    RaisePropertyChanged(() => AllItems);
                }
            }
        }

        private ValicMricMap _selectedItem;
        /// <summary>
        /// The currently selected ValicMricMap item
        /// </summary>
        public ValicMricMap SelectedItem
        {
            get { return _selectedItem; }
            set
            {
                if (_selectedItem != value)
                {
                    _selectedItem = value;
                    RaisePropertyChanged(() => SelectedItem);
                }
            }
        }

        private ObservableCollection<string> _states;
        public ObservableCollection<string> States
        {
            get { return _states; }
            set
            {
                if (_states != value)
                {
                    _states = value;
                    RaisePropertyChanged(() => States);
                }
            }
        }

        private string _state;
        public string State
        {
            get { return _state; }
            set
            {
                if (_state != value)
                {
                    _state = value;
                    RaisePropertyChanged(() => State);
                }
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Setup view model
        /// </summary>
        private void Setup()
        {
            AllItems = new ObservableCollection<ValicMricMap>();
            States = new ObservableCollection<string>();

            AddCommands();

            SampleData.CreateSampleData();
        }

        /// <summary>
        /// Adds the commands to the view model
        /// </summary>
        private void AddCommands()
        {
            LoadedCommand = new RelayCommand(() => LoadedCommandExecuted());
        }

        /// <summary>
        /// Occurs when user control is navigated to, reloaded etc.
        /// </summary>
        private void LoadedCommandExecuted()
        {
            SetupBackgroundWorker();
        }

        /// <summary>
        /// Sets up background worker for this VM
        /// </summary>
        private void SetupBackgroundWorker()
        {
            try
            {
                GlobalContractedEmployerInfo.VendorID = 32;  //32 = VALIC
                string addon;
                for (int j = 0; j < 10; j++)
                {
                    for (int i = 1; i < SampleData.completeCollection.Count; i++)
                    {
                        //App.Current.Dispatcher.Invoke((Action)delegate()
                        //{
                        Random rnd = new Random();
                        addon = rnd.Next(1000).ToString();
                        //ContractedEmployer ce=new ContractedEmployer(i*2, "FL", addon);
                        ValicMricMap vmm = new ValicMricMap(i, i, SampleData.completeCollection[i].StateAbbreviation, addon, SampleData.completeCollection[i], false);

                        AllItems.Add(vmm);
                        //}, DispatcherPriority.Background);
                    }
                }

                States.Add("FL");
                States.Add("MI");
            }
            catch
            {

            }
        }

        #endregion

        #region Commands

        /// <summary>
        /// Load Command
        /// </summary>
        public RelayCommand LoadedCommand
        {
            get;
            private set;
        }

        #endregion
    }
}