﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ConverterSlow
{
    public class ValicMricMap : NotificationObject, IDataErrorInfo
    {
        public ValicMricMap()
        { }

        public ValicMricMap(int? valicMricDistrictsID, int? employerID, string stateAbbreviation, string employerName, ContractedEmployer contractedEmployer, bool remove)
        {
            ValicMricDistrictsID = valicMricDistrictsID;
            EmployerID = employerID;
            StateAbbreviation = stateAbbreviation;
            EmployerName = employerName;
            ContractedEmployer = contractedEmployer;
            Remove = remove;
        }

        private int? _valicMricDistrictsID;
        public int? ValicMricDistrictsID
        {
            get { return _valicMricDistrictsID; }
            set
            {
                if (_valicMricDistrictsID != value)
                {
                    _valicMricDistrictsID = value;
                    FirePropertyChanged(() => ValicMricDistrictsID);
                }
            }
        }

        private int? _employerID;
        public int? EmployerID
        {
            get { return _employerID; }
            set
            {
                if (_employerID != value)
                {
                    _employerID = value;
                    FirePropertyChanged(() => EmployerID);
                }
            }
        }

        private string _stateAbbreviation;
        public string StateAbbreviation
        {
            get { return _stateAbbreviation; }
            set
            {
                if (_stateAbbreviation != value)
                {
                    _stateAbbreviation = value;
                    ContractedEmployer = null;
                    FirePropertyChanged(() => StateAbbreviation);
                }
            }
        }

        private string _employerName;
        public string EmployerName
        {
            get { return _employerName; }
            set
            {
                if (_employerName != value)
                {
                    _employerName = value;
                    FirePropertyChanged(() => EmployerName);
                }
            }
        }

        private ContractedEmployer _contractedEmployer;
        public ContractedEmployer ContractedEmployer
        {
            get { return _contractedEmployer; }
            set
            {
                if (_contractedEmployer != value)
                {
                    _contractedEmployer = value;

                    if (_contractedEmployer != null)
                    {
                        EmployerID = _contractedEmployer.EmployerID;
                    }
                    else
                    {
                        EmployerID = null;
                    }

                    FirePropertyChanged(() => ContractedEmployer);
                }
            }
        }

        private bool _remove;
        public bool Remove
        {
            get { return _remove; }
            set
            {
                if (_remove != value)
                {
                    _remove = value;
                    FirePropertyChanged(() => Remove);
                }
            }
        }

        private bool _isDirty;
        /// <summary>
        /// Bool tells whether item has changed or not
        /// </summary>
        public bool IsDirty
        {
            get { return _isDirty; }
            set
            {
                if (_isDirty != value)
                {
                    _isDirty = value;
                    FirePropertyChanged(() => IsDirty);
                }
            }
        }

        /// <summary>
        /// All properties that should be validated
        /// </summary>
        public readonly string[] ValidatedProperties = { "StateAbbreviation", "ContractedEmployer" };

        /// <summary>
        /// Bool tells whether item contains an error
        /// </summary>
        public bool IsValid
        {
            get
            {
                foreach (string property in ValidatedProperties)
                {
                    if (GetValidationError(property) != null) //there is an error
                        return false;
                }

                return true;
            }
        }

        private string GetValidationError(string propertyName)
        {
            string error = null;

            switch (propertyName)
            {
                case "StateAbbreviation":
                    {
                        if (String.IsNullOrWhiteSpace(StateAbbreviation))
                        {
                            error = "State Abbreviation is required";
                        }
                        break;
                    }
                case "ContractedEmployer":
                    {
                        if (ContractedEmployer == null)
                        {
                            error = "Employer is required";
                        }
                        else if (ContractedEmployer.EmployerID == null)
                        {
                            error = "Employer is required";
                        }
                        break;
                    }
                default:
                    {
                        error = null;
                        break;
                    }
            }

            return error;
        }

        /// <summary>
        /// Error message for validation
        /// </summary>
        /// <param name="columnName">Field name to be validated</param>
        /// <returns>error message</returns>
        public string this[string columnName]
        {
            get
            {
                string error = GetValidationError(columnName);

                return error;
            }
        }

        public string Error
        {
            get { return null; }
        }
    }
}
